@@ -68,7 +68,7 @@ fn bench_reissue_1_to_100(c: &mut Criterion) {
     let output_owner = bls_dkg_id();
     let output_owner_pk = output_owner.public_key_set.public_key();
 
-    let (reissue_tx, _) = sn_dbc::TransactionBuilder::default()
+    let reissue_tx = sn_dbc::TransactionBuilder::default()
         .add_input(genesis_dbc.clone(), genesis_secrets)
         .add_outputs((0..n_outputs).into_iter().map(|_| sn_dbc::Output {
             amount: 1,
@@ -79,6 +79,7 @@ fn bench_reissue_1_to_100(c: &mut Criterion) {
 
     let sig_share = genesis_owner
         .secret_key_share
+        .derive_child(&genesis_dbc.spend_key_index())
         .sign(&reissue_tx.blinded().hash());
 
     let sig = genesis_owner
@@ -88,10 +89,7 @@ fn bench_reissue_1_to_100(c: &mut Criterion) {
 
     let reissue = ReissueRequest {
         transaction: reissue_tx,
-        input_ownership_proofs: HashMap::from_iter([(
-            genesis_dbc.name(),
-            (genesis_owner.public_key_set.public_key(), sig),
-        )]),
+        input_ownership_proofs: HashMap::from_iter([(genesis_dbc.spend_key(), sig)]),
     };
 
     let spendbook = genesis.snapshot_spendbook();
@@ -101,7 +99,7 @@ fn bench_reissue_1_to_100(c: &mut Criterion) {
             genesis
                 .reissue(
                     black_box(reissue.clone()),
-                    black_box(BTreeSet::from_iter([genesis_dbc.name()])),
+                    black_box(BTreeSet::from_iter([genesis_dbc.spend_key()])),
                 )
                 .unwrap();
         })
@@ -116,7 +114,7 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
 
     let owners = Vec::from_iter((0..n_outputs).into_iter().map(|_| bls_dkg_id()));
 
-    let (reissue_tx, dbc_owners) = sn_dbc::TransactionBuilder::default()
+    let reissue_tx = sn_dbc::TransactionBuilder::default()
         .add_input(genesis_dbc.clone(), genesis_amount_secrets)
         .add_outputs(owners.iter().map(|owner| sn_dbc::Output {
             amount: 1,
@@ -125,17 +123,18 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         .build()
         .unwrap();
 
-    let dbc_owners = BTreeMap::from_iter(dbc_owners.into_iter().map(|(dbc_hash, owner_pk)| {
+    let dbc_owners = BTreeMap::from_iter(reissue_tx.outputs.iter().map(|out_dbc| {
         let owner = owners
             .iter()
-            .find(|o| o.public_key_set.public_key() == owner_pk)
+            .find(|o| o.public_key_set.public_key() == out_dbc.owner)
             .unwrap()
             .clone();
-        (dbc_hash, owner)
+        (out_dbc.owner, owner)
     }));
 
     let sig_share = genesis_owner
         .secret_key_share
+        .derive_child(&genesis_dbc.spend_key_index())
         .sign(&reissue_tx.blinded().hash());
 
     let sig = genesis_owner
@@ -143,10 +142,7 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         .combine_signatures(vec![(0, &sig_share)])
         .unwrap();
 
-    let input_ownership_proofs = HashMap::from_iter([(
-        genesis_dbc.name(),
-        (genesis_owner.public_key_set.public_key(), sig),
-    )]);
+    let input_ownership_proofs = HashMap::from_iter([(genesis_dbc.spend_key(), sig)]);
 
     let reissue = ReissueRequest {
         transaction: reissue_tx,
@@ -154,7 +150,10 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
     };
 
     let reissue_share = genesis
-        .reissue(reissue.clone(), BTreeSet::from_iter([genesis_dbc.name()]))
+        .reissue(
+            reissue.clone(),
+            BTreeSet::from_iter([genesis_dbc.spend_key()]),
+        )
         .unwrap();
 
     let (mint_key_set, mint_sig_share) = reissue_share
@@ -173,14 +172,14 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         content,
         transaction: reissue_share.dbc_transaction.clone(),
         transaction_sigs: BTreeMap::from_iter([(
-            genesis_dbc.name(),
+            genesis_dbc.spend_key(),
             (mint_key_set.public_key(), mint_sig.clone()),
         )]),
     }));
 
-    let (merge_tx, _) = sn_dbc::TransactionBuilder::default()
+    let merge_tx = sn_dbc::TransactionBuilder::default()
         .add_inputs(dbcs.iter().cloned().map(|dbc| {
-            let owner = &dbc_owners[&dbc.name()];
+            let owner = &dbc_owners[&dbc.owner()];
             let amount_secrets =
                 sn_dbc::DbcHelper::decrypt_amount_secrets(owner, &dbc.content).unwrap();
             (dbc, amount_secrets)
@@ -193,13 +192,16 @@ fn bench_reissue_100_to_1(c: &mut Criterion) {
         .unwrap();
 
     let input_ownership_proofs = HashMap::from_iter(dbcs.iter().map(|dbc| {
-        let owner = &dbc_owners[&dbc.name()];
-        let sig_share = owner.secret_key_share.sign(merge_tx.blinded().hash());
+        let owner = &dbc_owners[&dbc.owner()];
+        let sig_share = owner
+            .secret_key_share
+            .derive_child(&dbc.spend_key_index())
+            .sign(merge_tx.blinded().hash());
         let sig = owner
             .public_key_set
             .combine_signatures(vec![(0, &sig_share)])
             .unwrap();
-        (dbc.name(), (owner.public_key_set.public_key(), sig))
+        (dbc.spend_key(), sig)
     }));
 
     let merge_reissue = ReissueRequest {
