@@ -8,6 +8,7 @@
 // Software.
 
 //! Safe Network DBC Mint CLI playground.
+#![allow(clippy::from_iter_instead_of_collect)]
 
 use anyhow::{anyhow, Error, Result};
 use blsttc::poly::Poly;
@@ -20,9 +21,9 @@ use rustyline::error::ReadlineError;
 use rustyline::Editor;
 use serde::{Deserialize, Serialize};
 use sn_dbc::{
-    Amount, Dbc, DbcBuilder, DbcContent, DbcTransaction, Hash, Mint, NodeSignature, Output,
+    Amount, Dbc, DbcBuilder, DbcContent, DbcTransaction, Mint, NodeSignature, Output,
     ReissueRequest, ReissueTransaction, SimpleKeyManager as KeyManager, SimpleSigner as Signer,
-    SimpleSpendBook as SpendBook, TransactionBuilder,
+    SimpleSpendBook as SpendBook, SpendKey, TransactionBuilder,
 };
 use std::collections::{BTreeMap, BTreeSet, HashMap};
 use std::iter::FromIterator;
@@ -69,21 +70,21 @@ struct DbcUnblinded {
 #[derive(Debug, Clone, Serialize, Deserialize)]
 struct ReissueTransactionUnblinded {
     inner: ReissueTransaction,
-    inputs_owners: HashMap<Hash, PublicKeySet>,
-    outputs_owners: HashMap<Hash, PublicKeySet>,
+    input_pk_pks: HashMap<PublicKey, PublicKeySet>,
+    output_pk_pks: HashMap<PublicKey, PublicKeySet>,
 }
 
 /// A ReissueRequest with pubkey set for all the input and output Dbcs
 #[derive(Debug, Clone, Serialize, Deserialize)]
 struct ReissueRequestUnblinded {
     inner: ReissueRequest,
-    inputs_owners: HashMap<Hash, PublicKeySet>,
-    outputs_owners: HashMap<Hash, PublicKeySet>,
+    input_pk_pks: HashMap<PublicKey, PublicKeySet>,
+    output_pk_pks: HashMap<PublicKey, PublicKeySet>,
 }
 
 /// This type is just for serializing HashMap<Hash, <HashMap<usize, SignatureShare>>
 #[derive(Debug, Clone, Serialize, Deserialize, Default)]
-struct SignatureSharesMap(HashMap<Hash, HashMap<usize, SignatureShare>>);
+struct SignatureSharesMap(HashMap<PublicKey, HashMap<usize, SignatureShare>>);
 
 /// program entry point and interactive command handler.
 fn main() -> Result<()> {
@@ -236,7 +237,7 @@ fn mk_new_mint(secret_key_set: SecretKeySet, poly: Poly, amount: Amount) -> Resu
         .collect();
 
     // Todo: in a true multi-node mint, each node would call issue_genesis_dbc(), then the aggregated
-    // signatures would be combined here, so this mk_new_mint fn would to be broken apart.
+    // signatures would be combined here, so this mk_new_mint fn would be broken apart.
     let genesis_sig = secret_key_set
         .public_keys()
         .combine_signatures(node_sigs)
@@ -247,7 +248,7 @@ fn mk_new_mint(secret_key_set: SecretKeySet, poly: Poly, amount: Amount) -> Resu
         content: genesis_set[0].0.clone(),
         transaction: genesis_set[0].1.clone(),
         transaction_sigs: BTreeMap::from_iter([(
-            sn_dbc::GENESIS_DBC_INPUT,
+            sn_dbc::genesis_dbc_input(),
             (genesis_pubkey, genesis_sig),
         )]),
     };
@@ -389,8 +390,8 @@ fn print_mintinfo_human(mintinfo: &MintInfo) -> Result<()> {
     println!("\n");
 
     println!("-- SpendBook --\n");
-    for (dbchash, _tx) in &mintinfo.mintnode()?.spendbook {
-        println!("  {}", encode(&dbchash));
+    for (dbc_owner, _tx) in mintinfo.mintnode()?.spendbook.iter() {
+        println!("  {}", encode(&dbc_owner.0.to_bytes()));
     }
 
     println!();
@@ -412,7 +413,7 @@ fn print_dbc_human(
     outputs: bool,
     keys: Option<(&PublicKeySet, &BTreeMap<usize, SecretKeyShare>)>,
 ) -> Result<()> {
-    println!("id: {}\n", encode(dbc.inner.name()));
+    println!("id: {}\n", encode(dbc.inner.owner().to_bytes()));
 
     match keys {
         Some((public_key_set, secret_key_shares)) => {
@@ -441,13 +442,13 @@ fn print_dbc_human(
 
     println!("inputs:");
     for i in &dbc.inner.transaction.inputs {
-        println!("  {}", encode(i))
+        println!("  {}", encode(i.0.to_bytes()))
     }
 
     if outputs {
         println!("\noutputs:");
         for i in &dbc.inner.transaction.outputs {
-            println!("  {}", encode(i))
+            println!("  {}", encode(i.to_bytes()))
         }
     }
 
@@ -569,7 +570,7 @@ fn validate(mintinfo: &MintInfo) -> Result<()> {
     };
 
     match dbc.confirm_valid(mintinfo.mintnode()?.key_manager()) {
-        Ok(_) => match mintinfo.mintnode()?.is_spent(dbc.name())? {
+        Ok(_) => match mintinfo.mintnode()?.is_spent(dbc.spend_key())? {
             true => println!("\nThis DBC is unspendable.  (valid but has already been spent)\n"),
             false => println!("\nThis DBC is spendable.   (valid and has not been spent)\n"),
         },
@@ -584,7 +585,7 @@ fn prepare_tx() -> Result<()> {
     let mut tx_builder: TransactionBuilder = Default::default();
 
     // let mut inputs: HashSet<Dbc> = Default::default();
-    let mut inputs_owners: HashMap<Hash, PublicKeySet> = Default::default();
+    let mut input_pk_pks: HashMap<PublicKey, PublicKeySet> = Default::default();
     let mut pk_pks: HashMap<PublicKey, PublicKeySet> = Default::default();
 
     // Get DBC inputs from user
@@ -629,7 +630,7 @@ fn prepare_tx() -> Result<()> {
             .content
             .amount_secrets_by_secret_key_shares(&dbc.owner, &secrets)?;
 
-        inputs_owners.insert(dbc.inner.name(), dbc.owner);
+        input_pk_pks.insert(dbc.owner.public_key(), dbc.owner);
         tx_builder = tx_builder.add_input(dbc.inner, amount_secrets);
     }
 
@@ -688,19 +689,21 @@ fn prepare_tx() -> Result<()> {
 
     println!("\n\nThank-you.   Preparing ReissueTransaction...\n\n");
 
-    let (reissue_tx, output_owners) = tx_builder.build()?;
+    let reissue_tx = tx_builder.build()?;
 
     // generate output Hash -> PublicKeySet map
-    let mut outputs_owners: HashMap<Hash, PublicKeySet> = Default::default();
-    for (h, pk) in output_owners.iter() {
-        let pks = pk_pks.get(pk).ok_or_else(|| anyhow!("pubkey not found"))?;
-        outputs_owners.insert(*h, pks.clone());
+    let mut output_pk_pks: HashMap<PublicKey, PublicKeySet> = Default::default();
+    for out_dbc in reissue_tx.outputs.iter() {
+        let pks = pk_pks
+            .get(&out_dbc.owner)
+            .ok_or_else(|| anyhow!("pubkey not found"))?;
+        output_pk_pks.insert(out_dbc.owner, pks.clone());
     }
 
     let transaction = ReissueTransactionUnblinded {
         inner: reissue_tx,
-        inputs_owners,
-        outputs_owners,
+        input_pk_pks,
+        output_pk_pks,
     };
 
     println!("\n-- ReissueTransaction --");
@@ -723,13 +726,13 @@ fn sign_tx() -> Result<()> {
         println!(
             "Input #{} [id: {}, amount: ???  (encrypted)]",
             i,
-            encode(dbc.name()),
+            encode(dbc.owner().to_bytes()),
         );
         println!("-----------------");
 
         let pubkeyset = tx
-            .inputs_owners
-            .get(&dbc.name())
+            .input_pk_pks
+            .get(&dbc.owner())
             .ok_or_else(|| anyhow!("PubKeySet not found"))?;
 
         let mut secrets: HashMap<usize, SecretKeyShare> = Default::default();
@@ -764,10 +767,12 @@ fn sign_tx() -> Result<()> {
     for (dbc, secrets) in inputs.iter() {
         let mut sigs: HashMap<usize, SignatureShare> = Default::default();
         for (idx, secret) in secrets.iter() {
-            let sig_share = secret.sign(&tx.inner.blinded().hash());
+            let sig_share = secret
+                .derive_child(&dbc.spend_key_index())
+                .sign(&tx.inner.blinded().hash());
             sigs.insert(*idx, sig_share);
         }
-        sig_shares.0.insert(dbc.name(), sigs);
+        sig_shares.0.insert(dbc.owner(), sigs);
     }
 
     println!("\n-- SignatureSharesMap --");
@@ -781,19 +786,19 @@ fn sign_tx() -> Result<()> {
 fn prepare_reissue() -> Result<()> {
     let tx_input = readline_prompt_nl("\nReissueTransaction: ")?;
     let tx: ReissueTransactionUnblinded = from_be_hex(&tx_input)?;
-    let mut sig_shares_by_input: HashMap<Hash, BTreeMap<usize, SignatureShare>> =
+    let mut sig_shares_by_input: HashMap<PublicKey, BTreeMap<usize, SignatureShare>> =
         Default::default();
 
     // Get from user: SignatureSharesMap(s) for each tx input
     //                until required # of SignatureShare obtained.
     for dbc in tx.inner.inputs.iter() {
         println!("-----------------");
-        println!("Input [id: {}]", encode(dbc.name()),);
+        println!("Input [id: {}]", encode(dbc.owner().to_bytes()),);
         println!("-----------------");
 
         let pubkeyset = tx
-            .inputs_owners
-            .get(&dbc.name())
+            .input_pk_pks
+            .get(&dbc.owner())
             .ok_or_else(|| anyhow!("PubKeySet not found"))?;
 
         let mut num_shares = 0usize;
@@ -805,10 +810,10 @@ fn prepare_reissue() -> Result<()> {
             } else {
                 from_be_hex(&ssm_input)?
             };
-            for (name, shares) in shares_map.0.iter() {
+            for (owner, shares) in shares_map.0.iter() {
                 for (idx, share) in shares.iter() {
                     let list = sig_shares_by_input
-                        .entry(*name)
+                        .entry(*owner)
                         .or_insert_with(BTreeMap::default);
                     (*list).insert(*idx, share.clone());
                     num_shares += 1;
@@ -817,26 +822,26 @@ fn prepare_reissue() -> Result<()> {
         }
     }
 
-    let mut proofs: HashMap<Hash, (PublicKey, Signature)> = Default::default();
+    let mut proofs: HashMap<SpendKey, Signature> = Default::default();
     for dbc in tx.inner.inputs.iter() {
-        let shares = match sig_shares_by_input.get(&dbc.name()) {
+        let shares = match sig_shares_by_input.get(&dbc.owner()) {
             Some(s) => s,
             None => {
                 return Err(anyhow!(
                     "Signature Shares not found for input Dbc {}",
-                    encode(&dbc.name())
+                    encode(&dbc.owner().to_bytes())
                 ))
             }
         };
         let pubkeyset = tx
-            .inputs_owners
-            .get(&dbc.name())
+            .input_pk_pks
+            .get(&dbc.owner())
             .ok_or_else(|| anyhow!("PubKeySet not found"))?;
 
         let sig = pubkeyset
             .combine_signatures(shares)
             .map_err(|e| Error::msg(format!("{}", e)))?;
-        proofs.insert(dbc.name(), (pubkeyset.public_key(), sig));
+        proofs.insert(dbc.spend_key(), sig);
     }
 
     println!("\n\nThank-you.   Preparing ReissueRequest...\n\n");
@@ -846,8 +851,8 @@ fn prepare_reissue() -> Result<()> {
             transaction: tx.inner.clone(),
             input_ownership_proofs: proofs,
         },
-        inputs_owners: tx.inputs_owners.clone(),
-        outputs_owners: tx.outputs_owners,
+        input_pk_pks: tx.input_pk_pks,
+        output_pk_pks: tx.output_pk_pks,
     };
 
     println!("\n-- ReissueRequest --");
@@ -864,19 +869,20 @@ fn reissue(mintinfo: &mut MintInfo) -> Result<()> {
 
     println!("\n\nThank-you.   Generating DBC(s)...\n\n");
 
-    let input_hashes = reissue_request
-        .inner
-        .transaction
-        .inputs
-        .iter()
-        .map(|e| e.name())
-        .collect::<BTreeSet<_>>();
+    let input_keys = BTreeSet::from_iter(
+        reissue_request
+            .inner
+            .transaction
+            .inputs
+            .iter()
+            .map(Dbc::spend_key),
+    );
 
     reissue_exec(
         mintinfo,
         &reissue_request.inner,
-        &input_hashes,
-        &reissue_request.outputs_owners,
+        &input_keys,
+        &reissue_request.output_pk_pks,
     )
 }
 
@@ -887,17 +893,36 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
     let mut inputs_sks: HashMap<DbcUnblinded, BTreeMap<usize, SecretKeyShare>> = Default::default();
 
     // Get from user: input DBC(s) and required # of SecretKeyShare+index for each.
-    loop {
+    'input_loop: loop {
         println!("--------------");
         println!("Input DBC #{}", inputs_sks.len());
         println!("--------------\n");
 
-        let dbc_input = readline_prompt_nl("\nDBC Data, or 'done': ")?;
-        let dbc: DbcUnblinded = if dbc_input == "done" {
-            break;
-        } else {
-            from_be_hex(&dbc_input)?
-        };
+        let mut dbc: DbcUnblinded;
+        loop {
+            let dbc_input = readline_prompt_nl("\nDBC Data, or 'done': ")?;
+            if dbc_input == "done" {
+                break 'input_loop;
+            } else {
+                dbc = match from_be_hex(&dbc_input) {
+                    Ok(dbc) => dbc,
+                    Err(e) => {
+                        println!("DBC decoding failed: {:?}", e);
+                        continue;
+                    }
+                };
+                match dbc.inner.confirm_valid(mintinfo.mintnodes[0].key_manager()) {
+                    Ok(()) => {
+                        println!("Valid DBC");
+                        break;
+                    }
+                    Err(e) => {
+                        println!("Invalid DBC: {:?}", e);
+                        continue;
+                    }
+                }
+            };
+        }
 
         println!(
             "We need {} SecretKeyShare in order to decrypt the input amount.",
@@ -906,18 +931,38 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
 
         let mut secrets: BTreeMap<usize, SecretKeyShare> = Default::default();
         while secrets.len() < dbc.owner.threshold() + 1 {
-            let key = readline_prompt_nl("\nSecretKeyShare, or 'cancel': ")?;
-            let secret = if key == "cancel" {
-                println!("\nreissue_ez cancelled\n");
-                return Ok(());
-            } else {
-                from_be_hex(&key)?
-            };
-            let idx_input = readline_prompt("\nSecretKeyShare Index: ")?;
-            let idx: usize = idx_input.parse()?;
+            let mut secret: SecretKeyShare;
+            let idx: usize;
+            loop {
+                let key = readline_prompt_nl("\nSecretKeyShare, or 'cancel': ")?;
+
+                secret = if key == "cancel" {
+                    println!("\nreissue_ez cancelled\n");
+                    return Ok(());
+                } else {
+                    match from_be_hex(&key) {
+                        Ok(s) => s,
+                        Err(e) => {
+                            println!("Failed key decoding: {:?}", e);
+                            continue;
+                        }
+                    }
+                };
+
+                let idx_input = readline_prompt("\nSecretKeyShare Index: ")?;
+                idx = match idx_input.parse() {
+                    Ok(idx) => idx,
+                    Err(e) => {
+                        println!("index parsing failed {:?}", e);
+                        continue;
+                    }
+                };
+                break;
+            }
 
             secrets.insert(idx, secret);
         }
+
         let amount_secrets = dbc
             .inner
             .content
@@ -933,7 +978,7 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
     // note, we upcast to i128 to allow negative value.
     // This permits unbalanced inputs/outputs to reach sn_dbc layer for validation.
     let inputs_amount_sum = tx_builder.inputs_amount_sum();
-    while inputs_amount_sum as i128 - tx_builder.outputs_amount_sum() as i128 > 0 {
+    'output_loop: while inputs_amount_sum as i128 - tx_builder.outputs_amount_sum() as i128 > 0 {
         println!();
         println!("------------");
         println!("Output #{}", i);
@@ -961,14 +1006,21 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
             }
         }
 
-        let line = readline_prompt_nl("\nPublicKeySet, or 'cancel': ")?;
-        let pub_out = match line.as_str() {
-            "cancel" => break,
-            _ => line,
+        let pub_out_set: PublicKeySet = loop {
+            let line = readline_prompt_nl("\nPublicKeySet, or 'cancel': ")?;
+            let pub_out = match line.as_str() {
+                "cancel" => break 'output_loop,
+                _ => line,
+            };
+
+            match from_be_hex(&pub_out) {
+                Ok(pk_set) => break pk_set,
+                Err(e) => {
+                    println!("Failed PublicKeySet decoding {:?}", e);
+                    continue;
+                }
+            };
         };
-
-        let pub_out_set: PublicKeySet = from_be_hex(&pub_out)?;
-
         tx_builder = tx_builder.add_output(Output {
             amount,
             owner: pub_out_set.public_key(),
@@ -980,29 +1032,25 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
 
     println!("\n\nThank-you.   Generating DBC(s)...\n\n");
 
-    let input_hashes = tx_builder.inputs_hashes();
-    let (transaction, output_owners) = tx_builder.build()?;
-
-    // generate output Hash -> PublicKeySet map
-    let mut outputs_pks: HashMap<Hash, PublicKeySet> = Default::default();
-    for (h, pk) in output_owners.iter() {
-        let pks = pk_pks.get(pk).ok_or_else(|| anyhow!("pubkey not found"))?;
-        outputs_pks.insert(*h, pks.clone());
-    }
+    let input_owners = tx_builder.input_spend_keys();
+    let transaction = tx_builder.build()?;
+    let tx_hash = transaction.blinded().hash();
 
     // for each input Dbc, combine owner's SignatureShare(s) to obtain owner's Signature
-    let mut proofs: HashMap<Hash, (PublicKey, Signature)> = Default::default();
+    let mut proofs: HashMap<SpendKey, Signature> = Default::default();
     for (dbc, secrets) in inputs_sks.iter() {
         let mut sig_shares: BTreeMap<usize, SignatureShare> = Default::default();
         for (idx, secret) in secrets.iter() {
-            let sig_share = secret.sign(&transaction.blinded().hash());
+            let sig_share = secret
+                .derive_child(&dbc.inner.spend_key_index())
+                .sign(&tx_hash);
             sig_shares.insert(*idx, sig_share.clone());
         }
         let sig = dbc
             .owner
             .combine_signatures(&sig_shares)
             .map_err(|e| anyhow!(e))?;
-        proofs.insert(dbc.inner.name(), (dbc.owner.public_key(), sig));
+        proofs.insert(dbc.inner.spend_key(), sig);
     }
 
     let reissue_request = ReissueRequest {
@@ -1010,15 +1058,17 @@ fn reissue_ez(mintinfo: &mut MintInfo) -> Result<()> {
         input_ownership_proofs: proofs,
     };
 
-    reissue_exec(mintinfo, &reissue_request, &input_hashes, &outputs_pks)
+    println!("Prepared reissue request...");
+
+    reissue_exec(mintinfo, &reissue_request, &input_owners, &pk_pks)
 }
 
 /// Performs reissue
 fn reissue_exec(
     mintinfo: &mut MintInfo,
     reissue_request: &ReissueRequest,
-    input_hashes: &BTreeSet<Hash>,
-    outputs_pks: &HashMap<Hash, PublicKeySet>,
+    input_owners: &BTreeSet<SpendKey>,
+    output_pk_pks: &HashMap<PublicKey, PublicKeySet>,
 ) -> Result<()> {
     let mut dbc_builder: DbcBuilder = Default::default();
     dbc_builder = dbc_builder.set_reissue_transaction(reissue_request.transaction.clone());
@@ -1029,7 +1079,8 @@ fn reissue_exec(
     for mint in mintinfo.mintnodes.iter_mut() {
         // here we pretend the client has made a network request to a single mint node
         // so this mint.reissue() execs on the Mint node and returns data to client.
-        let reissue_share = mint.reissue(reissue_request.clone(), input_hashes.clone())?;
+        println!("Sending reissue request..");
+        let reissue_share = mint.reissue(reissue_request.clone(), input_owners.clone())?;
 
         // and now we are back to client code.
         dbc_builder = dbc_builder.add_reissue_share(reissue_share);
@@ -1039,8 +1090,8 @@ fn reissue_exec(
 
     // for each output, construct DbcUnblinded and display
     for dbc in output_dbcs.iter() {
-        let pubkeyset = outputs_pks
-            .get(&dbc.content.hash())
+        let pubkeyset = output_pk_pks
+            .get(&dbc.owner())
             .ok_or_else(|| anyhow!("PubKeySet not found"))?;
         let dbc_owned = DbcUnblinded {
             inner: dbc.clone(),
