@@ -5,14 +5,14 @@ use std::iter::FromIterator;
 use curve25519_dalek_ng::scalar::Scalar;
 
 use crate::{
-    Amount, AmountSecrets, Dbc, DbcContent, Error, Hash, NodeSignature, ReissueShare,
-    ReissueTransaction, Result,
+    Amount, AmountSecrets, Dbc, DbcContent, Error, NodeSignature, PublicKey, ReissueShare,
+    ReissueTransaction, Result, SpendKey,
 };
 
 ///! Unblinded data for creating sn_dbc::DbcContent
 pub struct Output {
     pub amount: Amount,
-    pub owner: blsttc::PublicKey,
+    pub owner: PublicKey,
 }
 
 #[derive(Default)]
@@ -42,11 +42,12 @@ impl TransactionBuilder {
         self
     }
 
-    pub fn inputs_hashes(&self) -> BTreeSet<Hash> {
-        self.inputs
-            .iter()
-            .map(|(dbc, _)| dbc.name())
-            .collect::<BTreeSet<_>>()
+    pub fn input_owners(&self) -> BTreeSet<PublicKey> {
+        BTreeSet::from_iter(self.inputs.keys().map(Dbc::owner))
+    }
+
+    pub fn input_spend_keys(&self) -> BTreeSet<SpendKey> {
+        BTreeSet::from_iter(self.inputs.keys().map(Dbc::spend_key))
     }
 
     pub fn inputs_amount_sum(&self) -> Amount {
@@ -57,8 +58,8 @@ impl TransactionBuilder {
         self.outputs.iter().map(|o| o.amount).sum()
     }
 
-    pub fn build(self) -> Result<(ReissueTransaction, HashMap<crate::Hash, blsttc::PublicKey>)> {
-        let parents = BTreeSet::from_iter(self.inputs.keys().map(Dbc::name));
+    pub fn build(self) -> Result<ReissueTransaction> {
+        let parents = BTreeSet::from_iter(self.inputs.keys().map(Dbc::spend_key));
         let inputs_bf_sum = self
             .inputs
             .values()
@@ -89,13 +90,8 @@ impl TransactionBuilder {
             .collect::<Result<Vec<_>>>()?;
 
         let inputs = HashSet::from_iter(self.inputs.into_keys());
-        let output_owners = HashMap::from_iter(
-            outputs_and_owners
-                .iter()
-                .map(|(dbc_content, owner)| (dbc_content.hash(), *owner)),
-        );
         let outputs = HashSet::from_iter(outputs_and_owners.into_iter().map(|(o, _)| o));
-        Ok((ReissueTransaction { inputs, outputs }, output_owners))
+        Ok(ReissueTransaction { inputs, outputs })
     }
 }
 
@@ -176,7 +172,7 @@ impl DbcBuilder {
 
             // Verify that each input has a NodeSignature
             for input in reissue_transaction.inputs.iter() {
-                if rs.mint_node_signatures.get(&input.name()).is_none() {
+                if rs.mint_node_signatures.get(&input.spend_key()).is_none() {
                     return Err(Error::ReissueShareMintNodeSignatureNotFoundForInput);
                 }
             }
@@ -216,7 +212,7 @@ impl DbcBuilder {
                     .iter()
                     .map(|input| {
                         (
-                            input.name(),
+                            input.spend_key(),
                             (mint_public_key_set.public_key(), mint_sig.clone()),
                         )
                     })
@@ -225,7 +221,7 @@ impl DbcBuilder {
             .collect();
 
         // sort outputs by name
-        output_dbcs.sort_by_key(|d| d.name());
+        output_dbcs.sort_by_key(Dbc::owner);
 
         Ok(output_dbcs)
     }
