@@ -7,22 +7,48 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::{
-    DbcContent, DbcContentHash, DbcTransaction, Error, Hash, KeyManager, PublicKey, Result,
-    Signature,
+    DbcContent, DbcTransaction, Error, KeyManager, PublicKey, Result, Signature, SpendKey,
 };
+
 use serde::{Deserialize, Serialize};
 use std::collections::BTreeMap;
+use tiny_keccak::{Hasher, Sha3};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Deserialize, Serialize)]
 pub struct Dbc {
     pub content: DbcContent,
     pub transaction: DbcTransaction,
-    pub transaction_sigs: BTreeMap<DbcContentHash, (PublicKey, Signature)>,
+    pub transaction_sigs: BTreeMap<SpendKey, (PublicKey, Signature)>,
 }
 
 impl Dbc {
-    pub fn name(&self) -> Hash {
-        self.content.hash()
+    /// Derive the (public) spend key for this DBC.
+    pub fn spend_key(&self) -> SpendKey {
+        let index = self.spend_key_index();
+        SpendKey(self.owner().derive_child(&index))
+    }
+
+    /// Read the DBC owner
+    pub fn owner(&self) -> PublicKey {
+        self.content.owner
+    }
+
+    /// Calculate the spend key index, this index is used to derive the spend key.
+    pub fn spend_key_index(&self) -> [u8; 32] {
+        let mut sha3 = Sha3::v256();
+
+        sha3.update(&self.content.hash().0);
+        sha3.update(&self.transaction.hash().0);
+
+        for (in_key, (mint_key, mint_sig)) in self.transaction_sigs.iter() {
+            sha3.update(&in_key.0.to_bytes());
+            sha3.update(&mint_key.to_bytes());
+            sha3.update(&mint_sig.to_bytes());
+        }
+
+        let mut hash = [0u8; 32];
+        sha3.finalize(&mut hash);
+        hash
     }
 
     // Check there exists a DbcTransaction with the output containing this Dbc
@@ -43,7 +69,7 @@ impl Dbc {
             Err(Error::MissingSignatureForInput)
         } else if self.transaction.inputs != self.content.parents {
             Err(Error::DbcContentParentsDifferentFromTransactionInputs)
-        } else if !self.transaction.outputs.contains(&self.content.hash()) {
+        } else if !self.transaction.outputs.contains(&self.owner()) {
             Err(Error::DbcContentNotPresentInTransactionOutput)
         } else {
             Ok(())
@@ -62,7 +88,7 @@ mod tests {
 
     use crate::tests::{NonZeroTinyInt, TinyInt};
     use crate::{
-        Amount, DbcBuilder, DbcHelper, KeyManager, Mint, ReissueRequest, SimpleKeyManager,
+        Amount, DbcBuilder, DbcHelper, Hash, KeyManager, Mint, ReissueRequest, SimpleKeyManager,
         SimpleSigner, SimpleSpendBook,
     };
 
@@ -84,7 +110,7 @@ mod tests {
     ) -> Result<ReissueRequest, Error> {
         let amount_secrets = DbcHelper::decrypt_amount_secrets(dbc_owner, &dbc.content)?;
 
-        let (reissue_tx, _) = crate::TransactionBuilder::default()
+        let reissue_tx = crate::TransactionBuilder::default()
             .add_input(dbc.clone(), amount_secrets)
             .add_outputs(
                 divide(amount_secrets.amount, n_ways).map(|amount| crate::Output {
@@ -96,6 +122,7 @@ mod tests {
 
         let sig_share = dbc_owner
             .secret_key_share
+            .derive_child(&dbc.spend_key_index())
             .sign(&reissue_tx.blinded().hash());
 
         let sig = dbc_owner
@@ -103,11 +130,9 @@ mod tests {
             .combine_signatures(vec![(dbc_owner.index, &sig_share)])
             .unwrap();
 
-        let dbc_owner_key = dbc_owner.public_key_set.public_key();
-
         let request = ReissueRequest {
             transaction: reissue_tx,
-            input_ownership_proofs: HashMap::from_iter([(dbc.name(), (dbc_owner_key, sig))]),
+            input_ownership_proofs: HashMap::from_iter([(dbc.spend_key(), sig)]),
         };
 
         Ok(request)
@@ -122,13 +147,13 @@ mod tests {
             DbcContent::random_blinding_factor(),
         )?;
 
-        let input_content_hashes = BTreeSet::from_iter([input_content.hash()]);
+        let input_owners = BTreeSet::from_iter([input_content.owner]);
 
         let dbc = Dbc {
             content: input_content,
             transaction: DbcTransaction {
                 inputs: BTreeSet::new(),
-                outputs: input_content_hashes,
+                outputs: input_owners,
             },
             transaction_sigs: Default::default(),
         };
@@ -180,7 +205,7 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_trans,
             transaction_sigs: BTreeMap::from_iter([(
-                crate::mint::GENESIS_DBC_INPUT,
+                crate::genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
@@ -211,7 +236,7 @@ mod tests {
             (dbc, amount_secrets)
         });
 
-        let (reissue_tx, _) = crate::TransactionBuilder::default()
+        let reissue_tx = crate::TransactionBuilder::default()
             .add_inputs(inputs)
             .add_output(crate::Output {
                 amount,
@@ -219,20 +244,18 @@ mod tests {
             })
             .build()?;
 
-        let sig_share = input_owner
-            .secret_key_share
-            .sign(&reissue_tx.blinded().hash());
-
-        let input_owner_key_set = input_owner.public_key_set.clone();
-        let sig = input_owner_key_set
-            .combine_signatures(vec![(input_owner.index, &sig_share)])
-            .unwrap();
-
         let input_ownership_proofs = HashMap::from_iter(reissue_tx.inputs.iter().map(|input| {
-            (
-                input.name(),
-                (input_owner_key_set.public_key(), sig.clone()),
-            )
+            let sig_share = input_owner
+                .secret_key_share
+                .derive_child(&input.spend_key_index())
+                .sign(&reissue_tx.blinded().hash());
+
+            let sig = input_owner
+                .public_key_set
+                .combine_signatures(vec![(input_owner.index, &sig_share)])
+                .unwrap();
+
+            (input.spend_key(), sig)
         }));
 
         let reissue_request = ReissueRequest {
@@ -268,7 +291,7 @@ mod tests {
                     // add some random parents
                     (0..n_add_random_parents.coerce())
                         .into_iter()
-                        .map(|_| rand::random()),
+                        .map(|_| rand::random::<SpendKey>()),
                 ),
         );
 
@@ -279,15 +302,16 @@ mod tests {
             DbcContent::random_blinding_factor(),
         )?;
 
-        let mut fuzzed_transaction_sigs: BTreeMap<Hash, (PublicKey, Signature)> = BTreeMap::new();
+        let mut fuzzed_transaction_sigs: BTreeMap<SpendKey, (PublicKey, Signature)> =
+            BTreeMap::new();
 
         // Add valid sigs
         fuzzed_transaction_sigs.extend(
             reissue_share
                 .mint_node_signatures
-                .iter()
+                .keys()
                 .take(n_valid_sigs.coerce())
-                .map(|(in_hash, _)| (*in_hash, (genesis_key, mint_sig.clone()))),
+                .map(|in_owner| (*in_owner, (genesis_key, mint_sig.clone()))),
         );
         let mut repeating_inputs = reissue_request
             .transaction
@@ -297,7 +321,7 @@ mod tests {
             // skip the valid sigs so that we don't immediately overwrite them
             .skip(n_valid_sigs.coerce());
 
-        // Valid mint signatures BUT signing wrong message
+        // Invalid mint signatures BUT signing correct message
         for _ in 0..n_wrong_signer_sigs.coerce() {
             if let Some(input) = repeating_inputs.next() {
                 let id = crate::bls_dkg_id();
@@ -310,8 +334,10 @@ mod tests {
                     .public_key_set
                     .combine_signatures(vec![trans_sig_share.threshold_crypto()])
                     .unwrap();
-                fuzzed_transaction_sigs
-                    .insert(input.name(), (id.public_key_set.public_key(), trans_sig));
+                fuzzed_transaction_sigs.insert(
+                    input.spend_key(),
+                    (id.public_key_set.public_key(), trans_sig),
+                );
             }
         }
 
@@ -326,13 +352,15 @@ mod tests {
                     .combine_signatures(vec![wrong_msg_sig.threshold_crypto()])
                     .unwrap();
 
-                fuzzed_transaction_sigs.insert(input.name(), (genesis_key, wrong_msg_mint_sig));
+                fuzzed_transaction_sigs
+                    .insert(input.spend_key(), (genesis_key, wrong_msg_mint_sig));
             }
         }
 
         // Valid mint signatures for inputs not present in the transaction
         for _ in 0..n_extra_input_sigs.coerce() {
-            fuzzed_transaction_sigs.insert(rand::random(), (genesis_key, mint_sig.clone()));
+            fuzzed_transaction_sigs
+                .insert(rand::random::<SpendKey>(), (genesis_key, mint_sig.clone()));
         }
 
         let dbc = Dbc {
@@ -347,10 +375,9 @@ mod tests {
 
         let dbc_amount = DbcHelper::decrypt_amount(&input_owner, &dbc.content)?;
 
-        println!("Validation Result: {:#?}", validation_res);
         match validation_res {
             Ok(()) => {
-                assert!(dbc.transaction.outputs.contains(&dbc.content.hash()));
+                assert!(dbc.transaction.outputs.contains(&dbc.owner()));
                 assert!(n_inputs.coerce::<u8>() > 0);
                 assert!(n_valid_sigs.coerce::<u8>() >= n_inputs.coerce::<u8>());
                 assert_eq!(dbc_amount, amount);
@@ -376,10 +403,10 @@ mod tests {
                     n_add_random_parents.coerce::<u8>() > 0 || n_drop_parents.coerce::<u8>() > 0
                 );
                 assert!(dbc.transaction.inputs != dbc.content.parents);
-                assert!(!dbc.transaction.outputs.contains(&dbc.content.hash()));
+                assert!(!dbc.transaction.outputs.contains(&dbc.owner()));
             }
             Err(Error::DbcContentNotPresentInTransactionOutput) => {
-                assert!(!dbc.transaction.outputs.contains(&dbc.content.hash()));
+                assert!(!dbc.transaction.outputs.contains(&dbc.owner()));
             }
             Err(Error::TransactionMustHaveAnInput) => {
                 assert_eq!(n_inputs.coerce::<u8>(), 0);
