@@ -40,8 +40,7 @@ pub enum Error {
     #[error("DBC already spent in transaction: {transaction:?}")]
     DbcAlreadySpent {
         transaction: crate::DbcTransaction,
-        transaction_sigs:
-            BTreeMap<crate::DbcContentHash, (crate::PublicKeySet, crate::NodeSignature)>,
+        transaction_sigs: BTreeMap<crate::SpendKey, (crate::PublicKeySet, crate::NodeSignature)>,
     },
     #[error("Genesis Input has already been spent in a different transaction")]
     GenesisInputAlreadySpent,
