@@ -9,13 +9,7 @@
 
 use serde::{Deserialize, Serialize};
 use std::fmt;
-use std::ops::Deref;
-#[cfg(test)]
-use tiny_keccak::{Hasher, Sha3};
-/// These typdefs are to simplify algorithm for now and will be removed for production.
-#[derive(Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord, Serialize, Deserialize)]
-pub struct Hash([u8; 32]);
-pub(crate) type DbcContentHash = Hash;
+
 mod builder;
 mod dbc;
 mod dbc_content;
@@ -23,11 +17,12 @@ mod dbc_transaction;
 mod error;
 mod key_manager;
 mod mint;
+mod spend_book;
 
 pub use crate::{
     builder::{DbcBuilder, Output, TransactionBuilder},
     dbc::Dbc,
-    dbc_content::{Amount, AmountSecrets, BlindedOwner, DbcContent},
+    dbc_content::{Amount, AmountSecrets, DbcContent},
     dbc_transaction::DbcTransaction,
     error::{Error, Result},
     key_manager::{
@@ -35,11 +30,15 @@ pub use crate::{
         SimpleSigner,
     },
     mint::{
-        Mint, MintNodeSignatures, ReissueRequest, ReissueShare, ReissueTransaction,
-        SimpleSpendBook, SpendBook, GENESIS_DBC_INPUT,
+        genesis_dbc_input, Mint, MintNodeSignatures, ReissueRequest, ReissueShare,
+        ReissueTransaction,
     },
+    spend_book::{SimpleSpendBook, SpendBook, SpendKey},
 };
 
+#[derive(Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord, Serialize, Deserialize)]
+pub struct Hash([u8; 32]);
+
 impl From<[u8; 32]> for Hash {
     fn from(val: [u8; 32]) -> Hash {
         Hash(val)
@@ -54,35 +53,12 @@ impl fmt::Debug for Hash {
     }
 }
 
-impl Deref for Hash {
-    type Target = [u8];
-
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
 impl AsRef<[u8]> for Hash {
-    #[inline]
     fn as_ref(&self) -> &[u8] {
         &self.0
     }
 }
 
-#[cfg(test)]
-use rand::distributions::{Distribution, Standard};
-
-#[cfg(test)]
-use rand::Rng;
-
-/// used when fuzzing DBC's in testing.
-#[cfg(test)]
-impl Distribution<Hash> for Standard {
-    fn sample<R: Rng + ?Sized>(&self, rng: &mut R) -> Hash {
-        Hash(rng.gen())
-    }
-}
-
 #[cfg(feature = "dkg")]
 pub fn bls_dkg_id() -> bls_dkg::outcome::Outcome {
     use std::collections::BTreeSet;
@@ -133,6 +109,8 @@ impl DbcHelper {
 
 #[cfg(test)]
 fn sha3_256(input: &[u8]) -> [u8; 32] {
+    use tiny_keccak::{Hasher, Sha3};
+
     let mut sha3 = Sha3::v256();
     let mut output = [0; 32];
     sha3.update(input);
