@@ -14,8 +14,8 @@
 // Outputs <= input value
 
 use crate::{
-    Amount, Dbc, DbcContent, DbcContentHash, DbcTransaction, Error, Hash, KeyManager,
-    NodeSignature, PublicKeySet, Result,
+    Amount, Dbc, DbcContent, DbcTransaction, Error, KeyManager, NodeSignature, PublicKey,
+    PublicKeySet, Result, SpendBook, SpendKey,
 };
 use curve25519_dalek_ng::ristretto::RistrettoPoint;
 use serde::{Deserialize, Serialize};
@@ -24,67 +24,12 @@ use std::{
     iter::FromIterator,
 };
 
-pub type MintNodeSignatures = BTreeMap<DbcContentHash, (PublicKeySet, NodeSignature)>;
+pub type MintNodeSignatures = BTreeMap<SpendKey, (PublicKeySet, NodeSignature)>;
 
-pub const GENESIS_DBC_INPUT: Hash = Hash([0u8; 32]);
-
-pub trait SpendBook: std::fmt::Debug + Clone {
-    type Error: std::error::Error;
-
-    fn lookup(&self, dbc_hash: &DbcContentHash) -> Result<Option<&DbcTransaction>, Self::Error>;
-    fn log(
-        &mut self,
-        dbc_hash: DbcContentHash,
-        transaction: DbcTransaction,
-    ) -> Result<(), Self::Error>;
-}
-
-#[derive(Debug, Default, Clone, Serialize, Deserialize)]
-pub struct SimpleSpendBook {
-    pub transactions: BTreeMap<DbcContentHash, DbcTransaction>,
-}
-
-impl SpendBook for SimpleSpendBook {
-    type Error = std::convert::Infallible;
-
-    fn lookup(&self, dbc_hash: &DbcContentHash) -> Result<Option<&DbcTransaction>, Self::Error> {
-        Ok(self.transactions.get(dbc_hash))
-    }
-
-    fn log(
-        &mut self,
-        dbc_hash: DbcContentHash,
-        transaction: DbcTransaction,
-    ) -> Result<(), Self::Error> {
-        self.transactions.insert(dbc_hash, transaction);
-        Ok(())
-    }
-}
-
-impl<'a> IntoIterator for &'a SimpleSpendBook {
-    type Item = (&'a DbcContentHash, &'a DbcTransaction);
-    type IntoIter = std::collections::btree_map::Iter<'a, DbcContentHash, DbcTransaction>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.transactions.iter()
-    }
-}
-
-impl IntoIterator for SimpleSpendBook {
-    type Item = (DbcContentHash, DbcTransaction);
-    type IntoIter = std::collections::btree_map::IntoIter<DbcContentHash, DbcTransaction>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.transactions.into_iter()
-    }
-}
-
-impl SimpleSpendBook {
-    pub fn new() -> Self {
-        Self {
-            transactions: Default::default(),
-        }
-    }
+pub fn genesis_dbc_input() -> SpendKey {
+    use blsttc::group::CurveProjective;
+    let gen_bytes = blsttc::convert::g1_to_be_bytes(blsttc::G1::one());
+    SpendKey(PublicKey::from_bytes(gen_bytes).unwrap())
 }
 
 #[derive(Eq, PartialEq, Debug, Clone, Deserialize, Serialize)]
@@ -96,8 +41,8 @@ pub struct ReissueTransaction {
 impl ReissueTransaction {
     pub fn blinded(&self) -> DbcTransaction {
         DbcTransaction {
-            inputs: BTreeSet::from_iter(self.inputs.iter().map(|i| i.name())),
-            outputs: BTreeSet::from_iter(self.outputs.iter().map(|i| i.hash())),
+            inputs: BTreeSet::from_iter(self.inputs.iter().map(Dbc::spend_key)),
+            outputs: BTreeSet::from_iter(self.outputs.iter().map(|i| i.owner)),
         }
     }
 
@@ -179,7 +124,7 @@ impl ReissueTransaction {
 pub struct ReissueRequest {
     pub transaction: ReissueTransaction,
     // Signatures from the owners of each input, signing `self.transaction.blinded().hash()`
-    pub input_ownership_proofs: HashMap<DbcContentHash, (blsttc::PublicKey, blsttc::Signature)>,
+    pub input_ownership_proofs: HashMap<SpendKey, blsttc::Signature>,
 }
 
 #[derive(Eq, PartialEq, Debug, Clone, Deserialize, Serialize)]
@@ -210,7 +155,7 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
         &mut self,
         amount: Amount,
     ) -> Result<(DbcContent, DbcTransaction, (PublicKeySet, NodeSignature))> {
-        let parents = BTreeSet::from_iter([GENESIS_DBC_INPUT]);
+        let parents = BTreeSet::from_iter([genesis_dbc_input()]);
         let content = DbcContent::new(
             parents,
             amount,
@@ -221,13 +166,13 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
             DbcContent::random_blinding_factor(),
         )?;
         let transaction = DbcTransaction {
-            inputs: BTreeSet::from_iter([GENESIS_DBC_INPUT]),
-            outputs: BTreeSet::from_iter([content.hash()]),
+            inputs: BTreeSet::from_iter([genesis_dbc_input()]),
+            outputs: BTreeSet::from_iter([content.owner]),
         };
 
         match self
             .spendbook
-            .lookup(&GENESIS_DBC_INPUT)
+            .lookup(&genesis_dbc_input())
             .map_err(|e| Error::SpendBook(e.to_string()))?
         {
             Some(tx) if tx != &transaction => return Err(Error::GenesisInputAlreadySpent),
@@ -235,7 +180,7 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
         }
 
         self.spendbook
-            .log(GENESIS_DBC_INPUT, transaction.clone())
+            .log(genesis_dbc_input(), transaction.clone())
             .map_err(|e| Error::SpendBook(e.to_string()))?;
         let transaction_sig = self
             .key_manager
@@ -254,10 +199,10 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
         ))
     }
 
-    pub fn is_spent(&self, dbc_hash: DbcContentHash) -> Result<bool> {
+    pub fn is_spent(&self, spend_key: SpendKey) -> Result<bool> {
         Ok(self
             .spendbook
-            .lookup(&dbc_hash)
+            .lookup(&spend_key)
             .map_err(|e| Error::SpendBook(e.to_string()))?
             .is_some())
     }
@@ -269,17 +214,16 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
     pub fn reissue(
         &mut self,
         reissue_req: ReissueRequest,
-        inputs_belonging_to_mint: BTreeSet<DbcContentHash>,
+        inputs_belonging_to_mint: BTreeSet<SpendKey>,
     ) -> Result<ReissueShare> {
         reissue_req.transaction.validate(self.key_manager())?;
         let transaction = reissue_req.transaction.blinded();
         let transaction_hash = transaction.hash();
 
         for input_dbc in reissue_req.transaction.inputs.iter() {
-            match reissue_req.input_ownership_proofs.get(&input_dbc.name()) {
-                Some((owner, sig)) if owner.verify(sig, &transaction_hash) => {
-                    input_dbc.content.validate_unblinding(owner)?;
-                }
+            let input_spend_key = input_dbc.spend_key();
+            match reissue_req.input_ownership_proofs.get(&input_spend_key) {
+                Some(sig) if input_spend_key.0.verify(sig, &transaction_hash) => (),
                 Some(_) => return Err(Error::FailedSignature),
                 None => return Err(Error::MissingInputOwnerProof),
             }
@@ -312,10 +256,10 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
             .transaction
             .inputs
             .iter()
-            .filter(|&i| inputs_belonging_to_mint.contains(&i.name()))
+            .filter(|&i| inputs_belonging_to_mint.contains(&i.spend_key()))
         {
             self.spendbook
-                .log(input.name(), transaction.clone())
+                .log(input.spend_key(), transaction.clone())
                 .map_err(|e| Error::SpendBook(e.to_string()))?;
         }
 
@@ -330,23 +274,20 @@ impl<K: KeyManager, S: SpendBook> Mint<K, S> {
     fn sign_transaction(
         &self,
         transaction: &DbcTransaction,
-    ) -> Result<BTreeMap<DbcContentHash, (PublicKeySet, NodeSignature)>> {
+    ) -> Result<BTreeMap<SpendKey, (PublicKeySet, NodeSignature)>> {
         let sig = self
             .key_manager
             .sign(&transaction.hash())
             .map_err(|e| Error::Signing(e.to_string()))?;
 
-        Ok(transaction
-            .inputs
-            .iter()
-            .copied()
-            .zip(std::iter::repeat((
+        Ok(BTreeMap::from_iter(
+            transaction.inputs.iter().copied().zip(std::iter::repeat((
                 self.key_manager
                     .public_key_set()
                     .map_err(|e| Error::Signing(e.to_string()))?,
                 sig,
-            )))
-            .collect())
+            ))),
+        ))
     }
 
     // Used in testing / benchmarking
@@ -368,7 +309,7 @@ mod tests {
 
     use crate::{
         tests::{TinyInt, TinyVec},
-        DbcBuilder, DbcHelper, SimpleKeyManager, SimpleSigner,
+        DbcBuilder, DbcHelper, SimpleKeyManager, SimpleSigner, SimpleSpendBook,
     };
 
     #[quickcheck]
@@ -393,7 +334,7 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_trans,
             transaction_sigs: BTreeMap::from_iter([(
-                GENESIS_DBC_INPUT,
+                genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
@@ -428,11 +369,11 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_tx,
             transaction_sigs: BTreeMap::from_iter([(
-                GENESIS_DBC_INPUT,
+                genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
-        let gen_dbc_name = genesis_dbc.name();
+        let gen_dbc_spend_key = genesis_dbc.spend_key();
 
         let genesis_amount_secrets =
             DbcHelper::decrypt_amount_secrets(&genesis_owner, &genesis_dbc.content)?;
@@ -440,8 +381,8 @@ mod tests {
         let output_owner = crate::bls_dkg_id();
         let output_owner_pk = output_owner.public_key_set.public_key();
 
-        let (reissue_tx, _output_owners) = crate::TransactionBuilder::default()
-            .add_input(genesis_dbc, genesis_amount_secrets)
+        let reissue_tx = crate::TransactionBuilder::default()
+            .add_input(genesis_dbc.clone(), genesis_amount_secrets)
             .add_outputs(output_amounts.iter().map(|a| crate::Output {
                 amount: *a,
                 owner: output_owner_pk,
@@ -450,6 +391,7 @@ mod tests {
 
         let sig_share = genesis_owner
             .secret_key_share
+            .derive_child(&genesis_dbc.spend_key_index())
             .sign(&reissue_tx.blinded().hash());
 
         let sig = genesis_owner
@@ -458,11 +400,11 @@ mod tests {
 
         let reissue_req = ReissueRequest {
             transaction: reissue_tx.clone(),
-            input_ownership_proofs: HashMap::from_iter([(gen_dbc_name, (genesis_key, sig))]),
+            input_ownership_proofs: HashMap::from_iter([(gen_dbc_spend_key, sig)]),
         };
 
         let reissue_share =
-            match genesis_node.reissue(reissue_req, BTreeSet::from_iter([gen_dbc_name])) {
+            match genesis_node.reissue(reissue_req, BTreeSet::from_iter([gen_dbc_spend_key])) {
                 Ok(rs) => {
                     // Verify that at least one output was present.
                     assert_ne!(n_outputs, 0);
@@ -514,17 +456,17 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_tx,
             transaction_sigs: BTreeMap::from_iter([(
-                GENESIS_DBC_INPUT,
+                genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
-        let gen_dbc_name = genesis_dbc.name();
+        let gen_dbc_spend_key = genesis_dbc.spend_key();
 
         let genesis_amount_secrets =
             DbcHelper::decrypt_amount_secrets(&genesis_owner, &genesis_dbc.content)?;
 
         let output_owner = crate::bls_dkg_id();
-        let (reissue_tx, _output_owners) = crate::TransactionBuilder::default()
+        let reissue_tx = crate::TransactionBuilder::default()
             .add_input(genesis_dbc.clone(), genesis_amount_secrets)
             .add_output(crate::Output {
                 amount: 1000,
@@ -534,7 +476,7 @@ mod tests {
 
         let sig_share = genesis_node
             .key_manager
-            .sign(&reissue_tx.blinded().hash())?;
+            .sign_with_child_key(&genesis_dbc.spend_key_index(), &reissue_tx.blinded().hash())?;
 
         let sig = genesis_node
             .key_manager
@@ -543,42 +485,42 @@ mod tests {
 
         let reissue_req = ReissueRequest {
             transaction: reissue_tx,
-            input_ownership_proofs: HashMap::from_iter([(gen_dbc_name, (genesis_key, sig))]),
+            input_ownership_proofs: HashMap::from_iter([(gen_dbc_spend_key, sig)]),
         };
 
         let reissue_share =
-            genesis_node.reissue(reissue_req, BTreeSet::from_iter([gen_dbc_name]))?;
+            genesis_node.reissue(reissue_req, BTreeSet::from_iter([gen_dbc_spend_key]))?;
         let t = reissue_share.dbc_transaction;
         let s = reissue_share.mint_node_signatures;
 
-        let (double_spend_reissue_tx, _output_owners) = crate::TransactionBuilder::default()
-            .add_input(genesis_dbc, genesis_amount_secrets)
+        let double_spend_reissue_tx = crate::TransactionBuilder::default()
+            .add_input(genesis_dbc.clone(), genesis_amount_secrets)
             .add_output(crate::Output {
                 amount: 1000,
                 owner: output_owner.public_key_set.public_key(),
             })
             .build()?;
 
-        let node_share = genesis_node
-            .key_manager
-            .sign(&double_spend_reissue_tx.blinded().hash())?;
+        let sig_share = genesis_node.key_manager.sign_with_child_key(
+            &genesis_dbc.spend_key_index(),
+            &double_spend_reissue_tx.blinded().hash(),
+        )?;
 
         let sig = genesis_node
             .key_manager
             .public_key_set()?
-            .combine_signatures(vec![node_share.threshold_crypto()])?;
+            .combine_signatures(vec![sig_share.threshold_crypto()])?;
 
         let double_spend_reissue_req = ReissueRequest {
             transaction: double_spend_reissue_tx,
-            input_ownership_proofs: HashMap::from_iter([(gen_dbc_name, (genesis_key, sig))]),
+            input_ownership_proofs: HashMap::from_iter([(gen_dbc_spend_key, sig)]),
         };
 
         let res = genesis_node.reissue(
             double_spend_reissue_req,
-            BTreeSet::from_iter([gen_dbc_name]),
+            BTreeSet::from_iter([gen_dbc_spend_key]),
         );
 
-        println!("res {:?}", res);
         assert!(matches!(
             res,
             Err(Error::DbcAlreadySpent { transaction, transaction_sigs }) if transaction == t && transaction_sigs == s
@@ -642,11 +584,11 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_tx,
             transaction_sigs: BTreeMap::from_iter([(
-                GENESIS_DBC_INPUT,
+                genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
-        let gen_dbc_name = genesis_dbc.name();
+        let gen_dbc_spend_key = genesis_dbc.spend_key();
 
         let genesis_amount_secrets =
             DbcHelper::decrypt_amount_secrets(&genesis_owner, &genesis_dbc.content)?;
@@ -656,8 +598,8 @@ mod tests {
             (owner.public_key_set.public_key(), (a, owner))
         }));
 
-        let (reissue_tx, output_owner_pks) = crate::TransactionBuilder::default()
-            .add_input(genesis_dbc, genesis_amount_secrets)
+        let reissue_tx = crate::TransactionBuilder::default()
+            .add_input(genesis_dbc.clone(), genesis_amount_secrets)
             .add_outputs(
                 owner_amounts_and_keys
                     .clone()
@@ -666,49 +608,45 @@ mod tests {
             )
             .build()?;
 
-        let owners =
-            BTreeMap::from_iter(output_owner_pks.into_iter().map(|(dbc_hash, owner_pk)| {
-                let (_, owner) = &owner_amounts_and_keys[&owner_pk];
-                (dbc_hash, owner)
-            }));
-
         let sig_share = genesis_node
             .key_manager
-            .sign(&reissue_tx.blinded().hash())?;
+            .sign_with_child_key(&genesis_dbc.spend_key_index(), &reissue_tx.blinded().hash())?;
         let sig = genesis_node
             .key_manager
             .public_key_set()?
             .combine_signatures(vec![sig_share.threshold_crypto()])?;
 
-        let reissue_req = ReissueRequest {
+        let reissue_req1 = ReissueRequest {
             transaction: reissue_tx,
-            input_ownership_proofs: HashMap::from_iter([(gen_dbc_name, (genesis_key, sig))]),
+            input_ownership_proofs: HashMap::from_iter([(gen_dbc_spend_key, sig)]),
         };
 
-        let reissue_share =
-            match genesis_node.reissue(reissue_req.clone(), BTreeSet::from_iter([gen_dbc_name])) {
-                Ok(rs) => {
-                    // Verify that at least one input (output in this tx) was present.
-                    assert!(!input_amounts.is_empty());
-                    rs
-                }
-                Err(Error::DbcReissueRequestDoesNotBalance) => {
-                    // Verify that no inputs (outputs in this tx) were present and we got correct validation error.
-                    assert!(input_amounts.is_empty());
-                    return Ok(());
-                }
-                Err(e) => return Err(e),
-            };
+        let reissue_share = match genesis_node.reissue(
+            reissue_req1.clone(),
+            BTreeSet::from_iter([gen_dbc_spend_key]),
+        ) {
+            Ok(rs) => {
+                // Verify that at least one input (output in this tx) was present.
+                assert!(!input_amounts.is_empty());
+                rs
+            }
+            Err(Error::DbcReissueRequestDoesNotBalance) => {
+                // Verify that no inputs (outputs in this tx) were present and we got correct validation error.
+                assert!(input_amounts.is_empty());
+                return Ok(());
+            }
+            Err(e) => return Err(e),
+        };
 
         // Aggregate ReissueShare to build output DBCs
-        let mut dbc_builder = DbcBuilder::new(reissue_req.transaction);
+        let mut dbc_builder = DbcBuilder::new(reissue_req1.transaction);
         dbc_builder = dbc_builder.add_reissue_share(reissue_share);
         let output_dbcs = dbc_builder.build()?;
 
         let input_dbcs = output_dbcs
             .into_iter()
             .map(|dbc| {
-                let owner = &owners[&dbc.name()];
+                let (_, owner) = &owner_amounts_and_keys[&dbc.owner()];
                 let amount_secrets = DbcHelper::decrypt_amount_secrets(owner, &dbc.content)?;
                 Ok((dbc, amount_secrets))
             })
@@ -716,7 +654,7 @@ mod tests {
 
         let outputs_owner = crate::bls_dkg_id();
 
-        let (mut reissue_tx, _) = crate::TransactionBuilder::default()
+        let mut reissue_tx = crate::TransactionBuilder::default()
             .add_inputs(input_dbcs)
             .add_outputs(output_amounts.iter().map(|amount| crate::Output {
                 amount: *amount,
@@ -734,7 +672,7 @@ mod tests {
                 extra_output_parents
                     .iter()
                     .filter(|idx| **idx == out_idx)
-                    .map(|_| rand::random::<Hash>()),
+                    .map(|_| rand::random::<SpendKey>()),
             );
             if !extra_random_parents.is_empty() {
                 dbcs_with_fuzzed_parents.insert(out_dbc_content.hash());
@@ -747,11 +685,14 @@ mod tests {
             .into_iter()
             .filter_map(|input_num| reissue_tx.inputs.iter().nth(input_num))
             .map(|dbc| {
-                let owner = &owners[&dbc.name()];
-                let sig_share = owner.secret_key_share.sign(&reissue_tx.blinded().hash());
+                let (_, owner) = &owner_amounts_and_keys[&dbc.owner()];
+                let sig_share = owner
+                    .secret_key_share
+                    .derive_child(&dbc.spend_key_index())
+                    .sign(&reissue_tx.blinded().hash());
                 let owner_key_set = &owner.public_key_set;
                 let sig = owner_key_set.combine_signatures(vec![(owner.index, &sig_share)])?;
-                Ok((dbc.name(), (owner_key_set.public_key(), sig)))
+                Ok((dbc.spend_key(), sig))
             })
             .collect::<Result<HashMap<_, _>, Error>>()?;
 
@@ -767,7 +708,7 @@ mod tests {
                 let sig =
                     owner_key_set.combine_signatures(vec![(random_owner.index, &sig_share)])?;
 
-                Ok((dbc.name(), (owner_key_set.public_key(), sig)))
+                Ok((dbc.spend_key(), sig))
             })
             .collect::<Result<HashMap<_, _>, Error>>()?;
 
@@ -785,14 +726,14 @@ mod tests {
             .collect::<Result<Vec<_>, _>>()?;
         let output_total_amount: Amount = dbc_output_amounts.iter().sum();
 
-        let reissue_req = ReissueRequest {
+        let reissue_req2 = ReissueRequest {
             transaction: reissue_tx,
             input_ownership_proofs,
         };
 
         let many_to_many_result = genesis_node.reissue(
-            reissue_req.clone(),
-            BTreeSet::from_iter(reissue_req.transaction.blinded().inputs),
+            reissue_req2.clone(),
+            BTreeSet::from_iter(reissue_req2.transaction.blinded().inputs),
         );
 
         match many_to_many_result {
@@ -801,16 +742,34 @@ mod tests {
                 assert_eq!(dbcs_with_fuzzed_parents.len(), 0);
                 assert!(
                     input_amounts.is_empty()
-                        || BTreeSet::from_iter(dbcs_with_invalid_ownership_proofs.keys())
-                            .intersection(&BTreeSet::from_iter(owners.keys()))
+                        || BTreeSet::from_iter(dbcs_with_invalid_ownership_proofs.keys().copied())
+                            .intersection(&BTreeSet::from_iter(owner_amounts_and_keys.keys().map(
+                                |pk| {
+                                    reissue_req2
+                                        .transaction
+                                        .inputs
+                                        .iter()
+                                        .find(|dbc| dbc.owner() == *pk)
+                                        .unwrap()
+                                        .spend_key()
+                                }
+                            )))
                             .next()
                             .is_none()
                 );
-                assert!(
-                    BTreeSet::from_iter(owners.keys()).is_subset(&BTreeSet::from_iter(
-                        dbcs_with_valid_ownership_proofs.keys()
-                    ))
-                );
+
+                assert!(BTreeSet::from_iter(owner_amounts_and_keys.keys().map(|pk| {
+                    reissue_req2
+                        .transaction
+                        .inputs
+                        .iter()
+                        .find(|dbc| dbc.owner() == *pk)
+                        .unwrap()
+                        .spend_key()
+                }))
+                .is_subset(&BTreeSet::from_iter(
+                    dbcs_with_valid_ownership_proofs.keys().copied()
+                )));
 
                 // The output amounts should correspond to the output_amounts
                 assert_eq!(
@@ -819,7 +778,7 @@ mod tests {
                 );
 
                 // Aggregate ReissueShare to build output DBCs
-                let mut dbc_builder = DbcBuilder::new(reissue_req.transaction);
+                let mut dbc_builder = DbcBuilder::new(reissue_req2.transaction);
                 dbc_builder = dbc_builder.add_reissue_share(rs);
                 let output_dbcs = dbc_builder.build()?;
 
@@ -856,8 +815,17 @@ mod tests {
             }
             Err(Error::MissingInputOwnerProof) => {
                 assert!(
-                    !BTreeSet::from_iter(owners.keys()).is_subset(&BTreeSet::from_iter(
-                        dbcs_with_valid_ownership_proofs.keys()
+                    !BTreeSet::from_iter(owner_amounts_and_keys.keys().map(|pk| {
+                        reissue_req2
+                            .transaction
+                            .inputs
+                            .iter()
+                            .find(|dbc| dbc.owner() == *pk)
+                            .unwrap()
+                            .spend_key()
+                    }))
+                    .is_subset(&BTreeSet::from_iter(
+                        dbcs_with_valid_ownership_proofs.keys().copied()
                     ))
                 );
             }
@@ -901,21 +869,25 @@ mod tests {
             input_owner.public_key_set.public_key(),
             DbcContent::random_blinding_factor(),
         )?;
-        let input_content_hashes = BTreeSet::from_iter([input_content.hash()]);
+        let input_owners = BTreeSet::from_iter([input_content.owner]);
+
+        let in_dbc = Dbc {
+            content: input_content,
+            transaction: DbcTransaction {
+                inputs: Default::default(),
+                outputs: input_owners,
+            },
+            transaction_sigs: Default::default(),
+        };
+
+        let in_dbc_spend_keys = BTreeSet::from_iter([in_dbc.spend_key()]);
 
         let fraudulant_reissue_result = genesis_node.reissue(
             ReissueRequest {
                 transaction: ReissueTransaction {
-                    inputs: HashSet::from_iter([Dbc {
-                        content: input_content,
-                        transaction: DbcTransaction {
-                            inputs: Default::default(),
-                            outputs: input_content_hashes.clone(),
-                        },
-                        transaction_sigs: Default::default(),
-                    }]),
+                    inputs: HashSet::from_iter([in_dbc]),
                     outputs: HashSet::from_iter([DbcContent::new(
-                        input_content_hashes.clone(),
+                        in_dbc_spend_keys.clone(),
                         100,
                         crate::bls_dkg_id().public_key_set.public_key(),
                         DbcContent::random_blinding_factor(),
@@ -923,7 +895,7 @@ mod tests {
                 },
                 input_ownership_proofs: HashMap::default(),
             },
-            input_content_hashes,
+            in_dbc_spend_keys,
         );
         assert!(fraudulant_reissue_result.is_err());
 
@@ -972,7 +944,7 @@ mod tests {
             content: gen_dbc_content,
             transaction: gen_dbc_tx,
             transaction_sigs: BTreeMap::from_iter([(
-                GENESIS_DBC_INPUT,
+                genesis_dbc_input(),
                 (genesis_key, genesis_sig),
             )]),
         };
@@ -984,7 +956,7 @@ mod tests {
         let outputs_owner_pk = outputs_owner.public_key_set.public_key();
         let output_amount = 1000;
 
-        let (mut transaction, _) = crate::TransactionBuilder::default()
+        let mut transaction = crate::TransactionBuilder::default()
             .add_input(genesis_dbc.clone(), genesis_secrets)
             .add_output(crate::Output {
                 amount: output_amount,
@@ -1019,9 +991,10 @@ mod tests {
         // Add the fudged output back into the reissue transaction.
         transaction.outputs.insert(out_dbc_content);
 
-        let sig_share = genesis_node
-            .key_manager
-            .sign(&transaction.blinded().hash())?;
+        let sig_share = genesis_node.key_manager.sign_with_child_key(
+            &genesis_dbc.spend_key_index(),
+            &transaction.blinded().hash(),
+        )?;
 
         let sig = genesis_node
             .key_manager
@@ -1030,10 +1003,7 @@ mod tests {
 
         let reissue_req = ReissueRequest {
             transaction,
-            input_ownership_proofs: HashMap::from_iter([(
-                genesis_dbc.name(),
-                (genesis_node.key_manager.public_key_set()?.public_key(), sig),
-            )]),
+            input_ownership_proofs: HashMap::from_iter([(genesis_dbc.spend_key(), sig)]),
         };
 
         // The mint should reissue this without error because the output commitment sum matches the
@@ -1044,7 +1014,7 @@ mod tests {
         // semi-unspendable state.
         let reissue_share = genesis_node.reissue(
             reissue_req.clone(),
-            BTreeSet::from_iter([genesis_dbc.name()]),
+            BTreeSet::from_iter([genesis_dbc.spend_key()]),
         )?;
 
         // Aggregate ReissueShare to build output DBCs
@@ -1095,7 +1065,7 @@ mod tests {
         let input_dbc = output_dbc;
         let input_secrets = DbcHelper::decrypt_amount_secrets(&outputs_owner, &input_dbc.content)?;
 
-        let (transaction, _) = crate::TransactionBuilder::default()
+        let transaction = crate::TransactionBuilder::default()
             .add_input(input_dbc.clone(), input_secrets)
             .add_output(crate::Output {
                 amount: input_secrets.amount,
@@ -1105,6 +1075,7 @@ mod tests {
 
         let sig_share = outputs_owner
             .secret_key_share
+            .derive_child(&input_dbc.spend_key_index())
             .sign(&transaction.blinded().hash());
 
         let sig = outputs_owner
@@ -1113,14 +1084,13 @@ mod tests {
 
         let reissue_req = ReissueRequest {
             transaction,
-            input_ownership_proofs: HashMap::from_iter([(
-                input_dbc.name(),
-                (outputs_owner.public_key_set.public_key(), sig),
-            )]),
+            input_ownership_proofs: HashMap::from_iter([(input_dbc.spend_key(), sig)]),
         };
 
         // The mint should give an error on reissue because the sum(inputs) does not equal sum(outputs)
-        let result = genesis_node.reissue(reissue_req, BTreeSet::from_iter([input_dbc.name()]));
+        let result =
+            genesis_node.reissue(reissue_req, BTreeSet::from_iter([input_dbc.spend_key()]));
+
         match result {
             Err(Error::DbcReissueRequestDoesNotBalance) => {}
             _ => panic!("Expecting Error::DbcReissueRequestDoesNotBalance"),
@@ -1130,7 +1100,7 @@ mod tests {
         // Phase 4. Successful reissue of mis-matched DBC using true committed amount.
         // ----------
 
-        let (transaction, _) = crate::TransactionBuilder::default()
+        let transaction = crate::TransactionBuilder::default()
             .add_input(input_dbc.clone(), input_secrets)
             .add_output(crate::Output {
                 amount: output_amount,
@@ -1140,6 +1110,7 @@ mod tests {
 
         let sig_share = outputs_owner
             .secret_key_share
+            .derive_child(&input_dbc.spend_key_index())
             .sign(&transaction.blinded().hash());
 
         let sig = outputs_owner
@@ -1148,14 +1119,12 @@ mod tests {
 
         let reissue_req = ReissueRequest {
             transaction,
-            input_ownership_proofs: HashMap::from_iter([(
-                input_dbc.name(),
-                (outputs_owner.public_key_set.public_key(), sig),
-            )]),
+            input_ownership_proofs: HashMap::from_iter([(input_dbc.spend_key(), sig)]),
         };
 
         // The mint should reissue without error because the sum(inputs) does equal sum(outputs)
-        let result = genesis_node.reissue(reissue_req, BTreeSet::from_iter([input_dbc.name()]));
+        let result =
+            genesis_node.reissue(reissue_req, BTreeSet::from_iter([input_dbc.spend_key()]));
         assert!(result.is_ok());
 
         Ok(())
